#include "imagelayer.h"
#include "texture.h"
#include "vertextable.h"
#include "nawiglcanvas.h"
#include "camera.h"
#include "nawiframe.h"
#include "fileinformation.h"
#include "downloadmanager.h"
#include <wx/filename.h>
#include <wx/glcanvas.h>

// Allocated in NawiApp
unsigned char *ImageLayer::coeffs = NULL;
Vector *ImageLayer::normals = NULL;

ImageLayer::ImageLayer(
	const wxString &name,
	Layer *parentLayer,
	const wxString &fileName,
	const wxString &url,
	const wxString &path,
	const Angle &minLatitude,
	const Angle &maxLatitude,
	const Angle &minLongitude,
	const Angle &maxLongitude,
	double worldRadius
	)
	:
	Layer(name, parentLayer->position, parentLayer->orientation, parentLayer),
	minLat(minLatitude),
	maxLat(maxLatitude),
	minLon(minLongitude),
	maxLon(maxLongitude),
	vertices(NULL),
	texture(0u),
	meshPointCount(32),
	worldRadius(worldRadius)
{
	fileInfo = new FileInformation(fileName, url, path);
}

ImageLayer::~ImageLayer()
{
	if(vertices)
		delete vertices;
	if(texture)
		delete texture;
}

void ImageLayer::initialize()
{
	wxFileName imageFile(fileInfo->path, fileInfo->fileName);

	if( imageFile.FileExists() )
	{
		if( !texture )
			texture = new Texture( imageFile.GetFullPath() );
		if( texture->cantBeLoaded )
		{
			delete texture;
			texture = NULL;
			return;
		}
		flagUpdatedLayers();
		createMesh();
		isInitialized = true;
		return;
	}
	else if( fileInfo->downloading )
		return;
	else if( !fileInfo->downloadFailed )
	{
		NawiFrame::downloadManager->add(fileInfo);
		return;
	}

	// No image available
	isOn = false;
	return;
}

void ImageLayer::update()
{
	if( !isInitialized )
		initialize();
}

void ImageLayer::createMesh()
{
	int upperBound = meshPointCount - 1;
	float scaleFactor = 1.f/upperBound;
	Angle latRange = Angle::abs(maxLat - minLat);
	Angle lonRange;
	if(minLon < maxLon)
		lonRange = maxLon - minLon;
	else
		lonRange = maxLon + Angle(2.*M_PI) - minLon;

	vertices = new VertexTable(
			meshPointCount * meshPointCount, // vertex count
			true,	// use normals
			false,	// use colors
			true	// use textures
			);
	for(int i = 0; i < meshPointCount; i++)
		for(int j = 0; j < meshPointCount; j++)
		{	
			vertices->setPosition(i*meshPointCount + j,
				Vector::sphericalToCartesian(
					worldRadius,
					maxLat - latRange * i * scaleFactor,
					minLon + lonRange * j * scaleFactor
					)
				);

			vertices->setTextureCoordinates(
				i*meshPointCount + j,
				Vector2(
					j*scaleFactor,
					(meshPointCount - 1 - i) * scaleFactor
					)
				);
		}

	vertices->setPrimitiveCount(2 * upperBound * upperBound);
	for(int i = 0; i < upperBound; i++)
	{
		for(int j = 0; j < upperBound; j++)
		{
			vertices->setPrimitive(
				2 * (i*upperBound + j),
				(short)(    i * meshPointCount + j),
				(short)((i+1) * meshPointCount + j),
				(short)(    i * meshPointCount + j+1)
				);

			vertices->setPrimitive(
				2 * (i*upperBound + j) + 1,
				(short)(    i * meshPointCount + j+1),
				(short)((i+1) * meshPointCount + j),
				(short)((i+1) * meshPointCount + j+1)
				);
		}
	}

	calculateNormals(vertices);
}

void ImageLayer::calculateNormals(VertexTable *table)
{
	// Average normals
	int i = 0;

	memset(coeffs, 0, sizeof(unsigned char) * table->vertexCount);

	// Avoid conversion to short during sum
	memset(normals, 0, sizeof(Vector) * table->vertexCount);

	for( i = 0; i < table->primitiveCount * 3; i += 3 )
	{
		Vector v1(
			table->getPosition( table->indices[i] ),
			table->getPosition( table->indices[i + 1] )
			);
		Vector v2(
			table->getPosition( table->indices[i] ),
			table->getPosition( table->indices[i + 2] )
			);
		Vector normal = v1 / v2; // cross

		normal.normalize();

		// Store the face's normal for each of the vertices that make up the face.
		normals[ table->indices[i  ] ] += normal;
		normals[ table->indices[i+1] ] += normal;
		normals[ table->indices[i+2] ] += normal;
		coeffs[ table->indices[i  ] ]++;
		coeffs[ table->indices[i+1] ]++;
		coeffs[ table->indices[i+2] ]++;
	}

	// Now loop through each vertex vector, and average out all the normals stored.
	for( i = 0; i < table->vertexCount; ++i )
		table->setNormal( i, normals[i] / coeffs[i] );
}

void ImageLayer::render(NawiGLCanvas *glCanvas)
{
	if(!isInitialized)
		return;

	if(!texture)
		return;

	vertices->texture = texture;

	glEnable(GL_DEPTH_TEST);
	glClear(GL_DEPTH_BUFFER_BIT);
	glDisable(GL_LIGHTING);

	glPushMatrix();
	glTranslatef(
		(float)-glCanvas->camera->referenceCenter.x,
		(float)-glCanvas->camera->referenceCenter.y,
		(float)-glCanvas->camera->referenceCenter.z
		);

	vertices->draw(
		true	// with texture
		);

	glPopMatrix();
		
	glDisable(GL_DEPTH_TEST);
}
