#include "matrix3x3.h"
#include "quaternion.h"
#include "vector.h"
#include <wx/math.h>

//! Copy constructor.
//! Allocates memory and copies the matrix contents.
Matrix3x3::Matrix3x3(const Matrix3x3 &m)
	: MatrixNxN(m)
{
}

Vector Matrix3x3::operator* (const Vector &v) const
{
	Vector res(0.,0.,0.);
	for( int i = 0; i < 3; i++ )
		for( int j = 0; j < 3; j++ )
			res[i] += (*this)(i, j) * v[j];
	return Vector(res);
}

void Matrix3x3::transpose()
{
	double tmp;

	tmp = operator()(1, 0);
	operator()(1, 0) = operator()(0, 1);
	operator()(0, 1) = tmp;

	tmp = operator()(2, 0);
	operator()(2, 0) = operator()(0, 2);
	operator()(0, 2) = tmp;

	tmp = operator()(1, 2);
	operator()(1, 2) = operator()(2, 1);
	operator()(2, 1) = tmp;
}

Vector Matrix3x3::multiply(const Matrix3x3 &m, const Vector &v)
{
	Vector res(0,0,0);
	for(int i = 0; i < 3; i++)
		for(int j = 0; j < 3; j++)
			res.val[i] += m(i,j) * v[j];
	return res;
}

//! Creates a rotation matrix from a quaternion
void Matrix3x3::setRotation(const Quaternion &orientation)
{
	Quaternion o = orientation;
	o.normalize();

	// First row
	operator()(0,0) = 1.0 - 2.0 * (o.y*o.y + o.z*o.z);
	operator()(0,1) = 2.0 * (o.x * o.y + o.z * o.w);
	operator()(0,2) = 2.0 * (o.x * o.z - o.y * o.w);

	// Second row
	operator()(1,0) = 2.0f * ( o.x * o.y - o.z * o.w );
	operator()(1,1) = 1.0f - 2.0f * ( o.x * o.x + o.z * o.z );
	operator()(1,2) = 2.0f * (o.z * o.y + o.x * o.w );

	// Third row
	operator()(2,0) = 2.0f * ( o.x * o.z + o.y * o.w );
	operator()(2,1) = 2.0f * ( o.y * o.z - o.x * o.w );
	operator()(2,2) = 1.0f - 2.0f * ( o.x * o.x + o.y * o.y );
}

void Matrix3x3::setRotation(double yaw, double pitch, double roll)
{
	double  siny = sin(yaw),
		cosy = cos(yaw),
		sinp = sin(pitch),
		cosp = cos(pitch),
		sinr = sin(roll),
		cosr = cos(roll);

	// First row
	operator()(0,0) = cosy * cosp;
	operator()(0,1) = cosy * sinp * sinr - siny * cosr;
	operator()(0,2) = cosy * sinp * cosr + siny * sinr;

	// Second row
	operator()(1,0) = siny * cosp;
	operator()(1,1) = siny * sinp * sinr + cosy * cosr;
	operator()(1,2) = siny * sinp * cosr - cosy * sinr;

	// Third row
	operator()(2,0) = -sinp;
	operator()(2,1) = cosp * sinr;
	operator()(2,2) = cosp * cosr;
}

Vector Matrix3x3::getCol(unsigned long col)
{
	if(col < size)
		return Vector( (Vector&)MatrixNxN::getCol(col) );
	else
		return Vector();
}

Vector Matrix3x3::getRow(unsigned long row)
{
	if(row < size)
		return Vector(
			operator()(row,0),
			operator()(row,1),
			operator()(row,2)
			);
	else
		return Vector();
}
