#include "nawiframe.h"
#include "world.h"
#include "updatethread.h"
#include "imagelayer.h"
#include "camera.h"
#include "nawiglcanvas.h"
#include "downloadmanager.h"
#include <wx/frame.h>

BEGIN_EVENT_TABLE(NawiFrame, wxFrame)
	EVT_CLOSE	(NawiFrame::onClose)
END_EVENT_TABLE()

wxString NawiFrame::directoryPath;
wxString NawiFrame::configPath = "config";
wxString NawiFrame::cachePath = "cache";
DownloadManager *NawiFrame::downloadManager;

NawiFrame::NawiFrame(NawiApp *parentApplication)
	:
	wxFrame(NULL, ID_ROOT_FRAME, "Nawi World Explorer"),
	parentApplication(parentApplication),
	updateThread(NULL),
	updateThreadTerminated(false),
	world(NULL)
{
	Show(); // Must be done before to create the canvas

	glCanvas = new NawiGLCanvas(this);

	// Allocate static tables, once for the app
	// (only one thread will use them.)
	ImageLayer::coeffs = new unsigned char[65535];
	ImageLayer::normals = new Vector[65535];

	world = new World(
			"Earth",
			6378137.
			);
	glCanvas->camera = new Camera(world->radius, glCanvas);

	int currentMonth = wxDateTime::GetCurrentMonth() - wxDateTime::Jan + 1;

	wxString path = NawiFrame::cachePath + wxString("/Earth/BmngBathy/");

	ImageLayer *bmngLayer = new ImageLayer(
		wxString::Format("Base Image - %02d.2004", currentMonth),
		world,
		wxString::Format("world.topo.bathy.2004%02d.jpg", currentMonth),
		"http://worldwind28.arc.nasa.gov/public/",
		path,
		-M_PI/2., M_PI/2., -M_PI, M_PI,
		world->radius);
	
	bmngLayer->isOn = true;
	world->children.Add(bmngLayer);
}

NawiFrame::~NawiFrame()
{
	if(updateThread)
		delete updateThread;
	if(world)
		delete world;
	if(glCanvas)
		delete glCanvas;

	// Delete static tables, allocated once for the app
	// (because there is only one thread using them.)
	if( ImageLayer::coeffs )
		delete[] ImageLayer::coeffs;
	if( ImageLayer::normals )
		delete[] ImageLayer::normals;
}

void NawiFrame::render()
{
	if( !glCanvas )
		return;

	long startTicks = 0;

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glClearColor(0., 0., 0., 0.);

	if (!world)
	{
		glCanvas->SwapBuffers();
		wxMilliSleep(25);
		return;
	}

	if (!updateThread && !updateThreadTerminated)
	{
		updateThread = new UpdateThread(this);
		updateThread->Create();
		updateThread->SetPriority(25);
		updateThread->Run();
	}

	glCanvas->camera->update();

	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	if( glCanvas->wireFrame )
		glPolygonMode(GL_FRONT, GL_LINE);
	else
		glPolygonMode(GL_FRONT, GL_FILL);

	world->updatedSinceLastRender = false;
	world->render(glCanvas);

	glDisable(GL_DEPTH_TEST);
}

void NawiFrame::onClose(wxCloseEvent &event)
{
	if(updateThread)
	{
		updateThread->keepUpdating = false;
		while( !updateThreadTerminated )
			wxMilliSleep(200);
		updateThread = NULL;
	}
	event.Skip();
}
